@@ -17,7 +17,7 @@ use HTML::DOM::Node 'DOCUMENT_NODE';
 use Scalar::Util 'weaken';
 use URI;
 
-our $VERSION = '0.056';
+our $VERSION = '0.057';
 our @ISA = 'HTML::DOM::Node';
 
 require    HTML::DOM::Collection;
@@ -45,7 +45,7 @@ HTML::DOM - A Perl implementation of the HTML Document Object Model
 
 =head1 VERSION
 
-Version 0.056 (alpha)
+Version 0.057 (alpha)
 
 B<WARNING:> This module is still at an experimental stage.  The API is 
 subject to change without
@@ -1437,7 +1437,8 @@ sub nodeName { '#document' }
 
 =item $tree->base
 
-Returns the base URL of the page; either from a <base href=...> tag or the
+Returns the base URL of the page; either from a <base href=...> tag, from
+the response object passed to C<new>, or the
 URL passed to C<new>.
 
 =cut
@@ -1449,15 +1450,37 @@ sub base {
 	){
 		return ''.$base_elem->attr('href');
 	}
-	else {
+	elsif (my $r = $$doc{_HTML_DOM_response}) {
+		my $base;
+		($base) = $r->header('Content-Base')
+		    or ($base) = $r->header('Content-Location')
+		    or $base = $r->header('Base');
+		# URI does not document $URI::scheme_re, but HTTP::Response
+		# (which is in a separate distribution) uses it.  It seems
+		# unlikely that it will go away in future URI versions, as
+		# that would break existing versions of HTTP::Response.
+		if ($base && $base =~ /^$URI::scheme_re:/o) {
+			# already absolute
+			return $base;
+		}
+		my $req = request $r;
+		my $uri = $req ? uri $req : $doc->URL;
+		return undef unless $uri;
+		# Work around URI bug.
+		if (!defined $base && $uri =~ /^[Dd][Aa][Tt][Aa]:/) {
+			return $uri;
+		}
 		no warnings 'uninitialized';
-		''.base{$$doc{_HTML_DOM_response}||return$doc->URL}
+		''.new_abs URI $base,$uri;
+	}
+	else {
+		$doc->URL
 	}
 }
 
 =item $tree->magic_forms
 
-This is mainly for internal use.  This returns a boolean indicating whether
+This is mainly for internal use.  It returns a boolean indicating whether
 the parser needed to associate formies with a form that did not contain
 them.  This happens when a closing </form> tag is missing and the form is
 closed implicitly, but a formie is encountered later.
